
from qgis.PyQt.QtCore import (QCoreApplication, QEvent, QFileInfo, QItemSelectionModel, QObject, QSettings, QSize, QSizeF, Qt, pyqtSignal)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QButtonGroup, QCheckBox, QComboBox, QDialog, QDialogButtonBox, QFileDialog, QGroupBox, QInputDialog,
                                 QListView, QMdiSubWindow, QMenu, QPushButton, QSizePolicy, QToolTip, QVBoxLayout)

from qgis.PyQt.QtGui import (QIcon, QKeySequence, QTextDocument)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)

from qgis.core import (QgsApplication, Qgis, QgsCoordinateReferenceSystem, QgsCoordinateTransform, QgsGeometry, QgsLayerTreeGroup,  
                       QgsLayerTreeModel, QgsPointXY, QgsProject, QgsRasterLayer, QgsRectangle, QgsSpatialIndex, QgsStyle, QgsVectorLayer)

from qgis.core import QgsMapLayerProxyModel
from qgis.gui import (QgsMapLayerComboBox, QgsLayerTreeMapCanvasBridge, QgsLayerTreeView,
                      QgsMapCanvas, QgsMapToolPan,  QgsMapToolZoom, QgsMessageBar,
                      QgsRendererPropertiesDialog, QgsRendererRasterPropertiesWidget, QgsRubberBand)


from os.path import (basename, dirname, exists, isfile, join, splitext)
from autoLoaderRaster.utils import (changeButtonIcon, cleanPath, fixeLabelsFileDialog, funnyPushButton, getThemeIcon, myPushButton, printerDlg, windowIcon)

from datetime import datetime as thedatetime



#---------------------------------------------------------------------------
# (c) Christophe MASSE 2013 - 2024
# revision code : 20-05-2020
# CLASS uiDoubleFocusGlassesPlugin - AUTOLOADERRASTER PLUGIN
#---------------------------------------------------------------------------

class uiDoubleFocusGlassesPlugin(QDialog):
    
    def __init__(self, parent):
        super(uiDoubleFocusGlassesPlugin, self).__init__()
        self.setObjectName("uiDoubleFocusGlassesPlugin")
        self.parent = parent
        self.styleSheetPushButton = self.parent.styleSheetPushButton
        self.styleComboBox = self.parent.styleComboBox
        self.resize(680, 680)
        self.setWindowFlags(Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
        self.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin", "Double glass viewer"))
        windowIcon(self, "checkdoubleglasses.png")

        self.iface = self.parent.iface
        self.saveFolder = join(dirname(__file__),"works")
        self.layers, self.currentLayers, self.vectorLayers = [], [], []
       
        self.canvas = self.parent.canvas
        self.rasterIcon = QIcon(getThemeIcon("raster.png"))
        self.imgChecked = getThemeIcon("checked.png").replace("\\","/")
        self.imgUnChecked = getThemeIcon("unchecked.png").replace("\\","/")        

        self.printMapview = QTextDocument()

        self.listLayers = QgsMapLayerComboBox(self)
        self.listLayers.setEditable(False)
        self.listLayers.setEnabled(True)
        self.listLayers.setFilters(QgsMapLayerProxyModel.RasterLayer)
        self.listLayers.model().sortOrder() 
        self.listLayers.setObjectName("listLayers")
        self.listLayers.setStyleSheet(self.styleComboBox)

        self.buttonToolBarNav = funnyPushButton(self) 
        self.buttonToolBarNav.initPushButton(25, 25, 40, 90, "buttonToolBarNav", "", QApplication.translate("autoLoaderRasterPlugin", "Hide/Show navigation toolbar"), True, getThemeIcon("navigation_on.png"), 25, 25, True)

        self.buttonToolBarLegend = funnyPushButton(self) 
        self.buttonToolBarLegend.initPushButton(25, 25, 40, 90, "buttonToolBarLegend", "", QApplication.translate("autoLoaderRasterPlugin", "Hide/Show legend toolbar"), True, getThemeIcon("legend_on.png"), 25, 25, True)

        toolTip = QApplication.translate("autoLoaderRasterPlugin","Actions ...")
        self.buttonActions = funnyPushButton(self) 
        self.buttonActions.initPushButton(40, 25, 40, 90, "buttonActions", "", toolTip, True, getThemeIcon("actions.png"), 40, 25, True)
        
        self.contextMnuActions(self.buttonActions)

        self.buttonToolBarNav.clicked.connect(self.getToolBarNav)
        self.buttonToolBarLegend.clicked.connect(self.getToolBarLegend)


        self.mapViewerLayer = QgsMapCanvas(self)
        self.refCrs = self.parent.projectAlr.crs()

        self.legende = actionsWindows(self.mapViewerLayer, "widgetConsole", "%s ..." % QApplication.translate("autoLoaderRasterPlugin","Legende"), 340, 250 )
        self.view = QgsLayerTreeView(self.legende)
        self.view.setGeometry(5, 30, 330, 215)

        styleSheet = "QgsLayerTreeView {opacity: 0.5;  background-color: rgba(255, 255, 255, 100);}"
        styleSheet+= "QgsLayerTreeView::indicator:checked {image: url(%s);}" % self.imgChecked
        styleSheet+= "QgsLayerTreeView::indicator:unchecked  {image: url(%s);}" % self.imgUnChecked
        self.view.setStyleSheet(styleSheet)
        
        self.initProject()
        self.view.show()

        self.widgetMapView = actionsWindows(self.mapViewerLayer, "widgetConsole", "%s ..." % QApplication.translate("autoLoaderRasterPlugin","Actions"), 340, 65 )
        ctrlMap(self, self.widgetMapView, True, False, False, 5, 0, 350, 30, False)
        self.widgetMapView.setGeometry(0, 0, 340, 65)
        self.legende.setGeometry(0, 65, 340, 250)


        self.toolPan = QgsMapToolPan(self.mapViewerLayer)
        self.toolZoomIn = QgsMapToolZoom(self.mapViewerLayer, False) 
        self.toolZoomOut = QgsMapToolZoom(self.mapViewerLayer, True)

        self.mapViewerLayer.setMapTool(self.toolZoomIn)
        contextMnu = self.setContextMenu()
        self.mapViewerLayer.setContextMenuPolicy(Qt.CustomContextMenu)
        self.mapViewerLayer.customContextMenuRequested.connect(self.contextMnuShortCut)

        try : self.mapViewerLayer.contextMenuAboutToShow.connect(self.addContextMnu)
        except : pass

        
        self.buttonAddPermanentLayers.clicked.connect(self.addLayersToMapCanvas)
        self.buttonDelPermanentLayers.clicked.connect(self.delLayersFromMapCanvas) 
        
        self.buttonCatchExtent.clicked.connect(self.catchExtent)
        self.buttonPan.clicked.connect(self.pan)
        self.buttonZoomIn.clicked.connect(self.zoomIn)
        self.buttonZoomOut.clicked.connect(self.zoomOut)

        self.contextMnuZoomAll(self.buttonZoomAll)
        self.buttonZoomPrevious.clicked.connect(self.zoomPrevious)
        self.buttonZoomNext.clicked.connect(self.zoomNext)
        self.buttonSendExtent.clicked.connect(self.sendExtentToMapCanvas)
        
        self.listLayersDouble = QComboBox(self) 
        self.listLayersDouble.setEditable(True)
        self.listLayersDouble.setEnabled(True)
        self.listLayersDouble.setObjectName("listLayersDouble")
        self.listLayersDouble.setStyleSheet(self.styleComboBox)

        self.updateListLayersDouble()

        self.view.doubleClicked.connect(self.setPropertiesLayer)
        self.mapViewerLayerDouble = QgsMapCanvas(self)
        self.mapViewerLayerDouble.setEnabled(False)

        self.mapViewerLayer.mapCanvasRefreshed.connect(self.setRefLayerForMapViewerDouble)
        
        self.listLayers.layerChanged.connect(self.getUpdateMapViewers)
        self.listLayersDouble.currentIndexChanged.connect(self.setRefLayerForMapViewerDouble)
        
        try :
          self.updateMapViewers(self.listLayers)
          self.getUpdateMapViewerDouble()
          self.synchronizeExtent()
        except : pass

        QgsProject(self.iface).instance().layersAdded.connect(self.updateListLayersDouble)
        QgsProject(self.iface).instance().layersRemoved.connect(self.updateListLayersDouble)
        QgsProject(self.iface).instance().layersWillBeRemoved.connect(self.clearProject) 
        self.iface.newProjectCreated.connect(self.clearProject)
        self.mapViewerLayer.extentsChanged.connect(self.synchronizeExtent)
        
    def updateComboLayers(self): self.listLayers.update()

    def getToolBarNav(self): self.setVisibility(self.widgetMapView, self.buttonToolBarNav, "navigation_%s.png")
    def getToolBarLegend(self): self.setVisibility(self.legende, self.buttonToolBarLegend, "legend_%s.png")
    
    def setVisibility(self, widget, actionTool, icon):
        if widget :
           widget.setVisible(not widget.isVisible ())
           cond = "on" if widget.isVisible() else "off"
        else : cond = "off"   
        actionTool.setIcon(QIcon(getThemeIcon(icon % cond)))


    def resizeEvent(self, event):
        centerX = int(self.width()/2)
        self.listLayers.setGeometry(5, 5, centerX-5-110, 25)
        self.buttonToolBarNav.setGeometry(centerX-5-100, 5, 25, 25)
        self.buttonToolBarLegend.setGeometry(centerX-75, 5, 25, 25)
        self.buttonActions.setGeometry(centerX-50, 5, 25, 25)
        self.listLayersDouble.setGeometry(centerX, 5, centerX-5, 25)
        self.mapViewerLayer.setGeometry(5, 30, centerX-5, self.height()-40)
        self.mapViewerLayerDouble.setGeometry(centerX, 30, centerX-5, self.height()-40)


    def setPropertiesLayer(self, index):
        layer = self.view.currentLayer()
        self.mapViewerLayer.setCurrentLayer(layer)
        
        if not layer : return
        if layer.type() == 0 : dlg = QgsRendererPropertiesDialog(layer, QgsStyle.defaultStyle(), False)
        elif layer.type() == 1 :
            dlg = QDialog()
            dlg.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowStaysOnTopHint | Qt.WindowCloseButtonHint)
            dlg.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin", "Properties for the layer"))
            layout = QVBoxLayout(dlg)
            dlg.widgetproperties = QgsRendererRasterPropertiesWidget(layer, self.mapViewerLayer, dlg)
            layout.addWidget(dlg.widgetproperties)
        
        else : return
        
        windowIcon(dlg , "layers.png")
        if layer.type() == 0 : dlg.setMapCanvas(self.mapViewerLayer)
        dlg.setWindowTitle("%s : %s" % (dlg.windowTitle(), layer.name()))
        buttonBox = None
        for conteneur in dlg.children():
                if conteneur.children()!= []:
                   for child in conteneur.children():
                          for elt in child.children():
                                  if type(elt) == QDialogButtonBox :
                                             buttonBox = elt
                                             break

        if buttonBox and layer.type() == 0 :
                self.applystyle = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin", "Apply style"), QDialogButtonBox.ActionRole)
                self.applystyle.clicked.connect(self.apply_style)
                buttonBox.rejected.connect(dlg.close)
        else :
           buttonBox = QDialogButtonBox()
           self.quitButton = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin","Cancel"), QDialogButtonBox.RejectRole)
           self.applystyle = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin", "Apply style"), QDialogButtonBox.ActionRole)
           self.acceptButton = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin","Validate"), QDialogButtonBox.AcceptRole)
           buttonBox.accepted.connect(self.apply_style_accept)
           buttonBox.rejected.connect(dlg.reject)
           layout.addWidget(buttonBox)
           self.applystyle.clicked.connect(self.apply_style)

        self.styleDlg = dlg                          
        dlg.exec_()
        self.mapViewerLayer.refresh()


    def apply_style_accept(self) :
        if self.styleDlg:
           self.styleDlg.widgetproperties.apply()
           self.mapViewerLayer.refresh()
           self.styleDlg.accept()

    def apply_style(self):
        if self.styleDlg:
                if self.mapViewerLayer.currentLayer().type()==0 : self.styleDlg.apply()
                else : self.styleDlg.widgetproperties.apply()
                self.mapViewerLayer.refresh()
                self.mapViewerLayerDouble.refresh()


    def clearProject(self):
        self.layers, self.currentLayers, self.vectorLayers  = [], [], []

        self.mapViewerLayer.blockSignals(True)

        if self.project : self.project.addMapLayers([])
        self.listLayers.clear()
        self.view.selectionModel().clear()
        self.initProject()

        self.mapViewerLayer.setLayers([])
        self.mapViewerLayer.refresh()

        self.mapViewerLayerDouble.setLayers([])
        self.mapViewerLayerDouble.refresh()

        self.mapViewerLayer.blockSignals(False)

        
        
    def initProject(self):
        self.project = QgsProject(self)
        crs = QSettings().value("Projections/layerDefaultCrs")
        self.crs = QgsCoordinateReferenceSystem(crs)
                                      
        self.project.setCrs(self.crs)
        self.project.addMapLayers([])

        self.model = QgsLayerTreeModel(self.project.layerTreeRoot())
        self.model.setFlag(QgsLayerTreeModel.AllowNodeChangeVisibility)
        self.model.setFlag(QgsLayerTreeModel.ShowLegend)
        self.view.setModel(self.model)


    def loadProject(self, layers):
        self.project.addMapLayers(layers)
        self.bridge = QgsLayerTreeMapCanvasBridge(self.project.layerTreeRoot(), self.mapViewerLayer)
        

    def contextMnuZoomAll(self, button):
        contextMenu = QMenu()

        action = QAction(QIcon(getThemeIcon("zoomtolayer.png")), QApplication.translate("autoLoaderRasterPlugin", "Zoom to raster layer extent"), self)
        action.setObjectName("zoomallrasterlayer")
        contextMenu.addAction(action)
        action.triggered.connect(self.zoomAllLayerRaster)

        action = QAction(QIcon(getThemeIcon("zoommap.png")), QApplication.translate("autoLoaderRasterPlugin", "Zoom all the map window"), self)
        action.setObjectName("zoomallmap")
        contextMenu.addAction(action)
        action.triggered.connect(self.zoomAllMap)

        button.setMenu(contextMenu)
        

    def contextMnuActions(self, button):  self.getContextMnu(self.sender(), None, button)
    def contextMnuShortCut(self, point): self.getContextMnu(self.sender(), point, None)
    def getContextMnu(self, sender, point, button):  
        if point :
          if not (sender == self.mapViewerLayer) : return
        contextMnu = self.setContextMenu()
        if point : contextMnu.exec_(self.sender().mapToGlobal(point))
        else : button.setMenu(contextMnu) 
         
    def setContextMenu(self):
        
        actions = {"prinMapAsImages" : ("print.png", "Print the maps", "" , self.printMaps, True),
                   "~0" : (),            
                   "saveMapAsImages" : ("savefile.png", "Save maps as images", "" , self.saveMapsAsImage, True)
                   }

        contextMnu = QMenu() 
        for key in actions :
            if key.startswith("~") : contextMnu.addSeparator()
            else :   
                  icon = getThemeIcon(actions[key][0])
                  action = QAction(QIcon(icon), QApplication.translate("autoLoaderRasterPlugin", actions[key][1]).replace(":",""), self)
                  if actions[key][2] != "" : action.setShortcut(QKeySequence(actions[key][2]))
                  setattr(contextMnu, key, action)
                  action.setEnabled(actions[key][4])
                  action.setObjectName(key)
                  contextMnu.addAction(action)
                  action.triggered.connect(actions[key][3])

        return contextMnu

    def addContextMnu(self, mapCanvasContextMnu) :
        contextMnu = self.setContextMenu()
        mapCanvasContextMnu.addMenu(contextMnu)
        

    def saveMapsAsImage(self):
        title = QApplication.translate("autoLoaderRasterPlugin", "Select the folder")
        
        dlgFile = QFileDialog(self, title)
        dlgFile.setDirectory(self.saveFolder)
        dlgFile.setFileMode(QFileDialog.DirectoryOnly)

        fixeLabelsFileDialog(self, dlgFile, 0, False)      

        if dlgFile.exec_():
           self.saveFolder = dlgFile.selectedFiles()[0].replace("\\","/")
           fileName, fileNameDouble = self.createFilesImages(self.saveFolder)

    def createFilesImages(self, folder):
        fileName = join(folder, "mapviewer1.png") 
        self.mapViewerLayer.saveAsImage(fileName, None, "PNG")

        fileNameDouble = join(folder ,"mapviewer2.png") 
        self.mapViewerLayerDouble.saveAsImage(fileNameDouble, None, "PNG")

        return fileName, fileNameDouble

    def printMaps(self):
        printer = QPrinter(); printer.setPageSize(QPrinter.A4); printer.setOrientation(QPrinter.Landscape);
        printer.setPageMargins(5, 5, 5, 5, QPrinter.Millimeter); printer.setOutputFormat(QPrinter.NativeFormat);

        fileName, fileNameDouble = self.createFilesImages(join(dirname(__file__), "works"))
       
        self.printMapview.clear()
        self.printMapview.setPageSize(QSizeF(printer.pageRect().size()))
       

        try:
            from osgeo.gdal import Open as gdalopen
            from osgeo.gdal import GA_ReadOnly
        except:
            from gdal import Open as gdalopen
            from gdal import GA_ReadOnly

        sourceWidth, sourceHeight = printer.pageRect().width(), printer.pageRect().height()
        dataset = gdalopen( fileName, GA_ReadOnly )
        orientation = QPrinter.Landscape
        
        if dataset :
            imgWidth, imgHeight = dataset.RasterXSize, dataset.RasterYSize
            if imgHeight > (imgWidth*1.1) : orientation = QPrinter.Portrait


        if orientation == QPrinter.Portrait :  scale = sourceHeight*0.8 / float(imgHeight)
        else :  scale = sourceWidth / float(imgWidth*2)

        
        imgWidth, imgHeight = int(imgWidth*scale), int(imgHeight*scale)

        orientation = QPrinter.Landscape
       
        title = "%s : " % QApplication.translate("autoLoaderRasterPlugin", "Print")

        html = """
        <head><title>Report</title></head>
        <body>
            <table width="100%">
                <tr><td><span style='font-size:1.2em'>{}</span><br><small>{}<br>{}</small></td></tr>
            </table>
          <hr>
          <table width="100%">
          <tr>
            <td><img src="{}" width="{}" height="{}"></td>
            <td><img src="{}" width="{}" height="{}"></td>
          </tr>
          </table>
          <hr>
          <p align=right>AutoLaoderRaster, le {}</p>
        </body>
        """.format(title, fileName, fileNameDouble, fileName, imgWidth, imgHeight, fileNameDouble, imgWidth, imgHeight, thedatetime.now().strftime("%d/%m/%Y à %Hh%M"))
       
        self.printMapview.setHtml(html)        
        dlgPrint = printerDlg(self, self.printMapview, "%s : " % QApplication.translate("autoLoaderRasterPlugin", "Print"), orientation)


    def catchExtent(self):
        indexListLayers = self.listLayers.currentIndex()
        
        extent = self.canvas.extent()
        coords = [[ QgsPointXY(extent.xMinimum(),extent.yMaximum()), QgsPointXY(extent.xMaximum(),extent.yMaximum()),
                                  QgsPointXY(extent.xMaximum(),extent.yMinimum()), QgsPointXY(extent.xMinimum(),extent.yMinimum()),
                                  QgsPointXY(extent.xMinimum(),extent.yMaximum()), ]]  
        geomTargetExtent = QgsGeometry().fromPolygonXY(coords)

        overLaps, favExtentIndex, indexes = 0, 0, []
        
        for i in range(self.listLayers.count()):
              layer = self.listLayers.layer(i)
              extent = layer.extent()

              coords = [[ QgsPointXY(extent.xMinimum(),extent.yMaximum()), QgsPointXY(extent.xMaximum(),extent.yMaximum()),
                                  QgsPointXY(extent.xMaximum(),extent.yMinimum()), QgsPointXY(extent.xMinimum(),extent.yMinimum()),
                                  QgsPointXY(extent.xMinimum(),extent.yMaximum()), ]]  
              geomExtent = QgsGeometry().fromPolygonXY(coords)

              if geomExtent.intersects(geomTargetExtent) :
                  area = geomExtent.intersection(geomTargetExtent).area()
                  if area > 0 :
                     indexes.append(i)
                     if area > overLaps : overLaps, favExtentIndex = area, i
 
        self.listLayers.setCurrentIndex(indexListLayers) if indexListLayers in indexes else self.listLayers.setCurrentIndex(favExtentIndex)
        self.getUpdateMapViewers()
        self.mapViewerLayer.setExtent(self.canvas.extent())
        

    def sendExtentToMapCanvas(self): 
        try :
           self.canvas.setExtent(self.mapViewerLayer.extent())
           self.canvas.refresh()
        except : pass

    def zoomIn(self): self.mapViewerLayer.setMapTool(self.toolZoomIn) 
    def zoomOut(self): self.mapViewerLayer.setMapTool(self.toolZoomOut)
    def zoomAllLayerRaster(self):
        extent = None
        for layer in self.mapViewerLayer.layers():
            if type(layer) == QgsRasterLayer :
               if not extent : extent = layer.dataProvider().extent()
               else : extent.combineExtentWith(layer.dataProvider().extent())
        if extent : self.mapViewerLayer.setExtent(extent)
        
            
    def zoomAllMap(self): self.mapViewerLayer.zoomToFullExtent()
    def zoomPrevious(self): self.mapViewerLayer.zoomToPreviousExtent()
    def zoomNext(self): self.mapViewerLayer.zoomToNextExtent()
    def pan(self): self.mapViewerLayer.setMapTool(self.toolPan)
        
    def getUpdateMapViewers(self): self.updateMapViewers(self.sender())
      
    def addVectoLayers(self):
        layers = list(self.mapViewerLayer.layers())
        layers.reverse()
        for layer in layers :
            if type(layer) in ( QgsVectorLayer, QgsRasterLayer) :  self.currentLayers.insert(0, layer)


    def setExtent(self): 
        try :
           extent = self.parent.matchTransform(self.mapViewerLayer.extent(), QgsProject(self.iface).instance().crs(), self.refCrs, QgsCoordinateTransform.ForwardTransform)
           self.mapViewerLayerDouble.setExtent(extent) 
           self.mapViewerLayerDouble.refresh()
        except : pass

        
    def setRefLayerForMapViewerDouble(self):
        if self.listLayersDouble.count() == 0: return
      
        layer = None
        
        for layer in self.layers :
            try :
                if layer.name() == self.listLayersDouble.currentText() :  self.setLayers(layer); self.refCrs = layer.crs() ; break
            except : pass
                
        if layer :
            self.setLayers(layer)
            self.mapViewerLayerDouble.setLayers([])
            self.addVectoLayers()
            try :
                if self.currentLayers :
                   
                   layers = [layer for layer in self.currentLayers if layer.type() == 0 or (layer.type() == 1 and not layer in self.mapViewerLayer.layers())]
                        
                   self.mapViewerLayerDouble.setLayers(layers) 
            except : pass


    def getUpdateMapViewerDouble(self):
        if self.currentLayers : self.mapViewerLayerDouble.setLayers(self.currentLayers)


    def updateMapViewers(self, qgsMapLayerComboBox):
    
        if qgsMapLayerComboBox == self.listLayersDouble : target = self.mapViewerLayerDouble  
        else : qgsMapLayerComboBox = self.listLayers; target = self.mapViewerLayer

        if not target : return
        if qgsMapLayerComboBox.count() == 0 : return
        
        layerRaster = self.listLayers.layer(qgsMapLayerComboBox.currentIndex())
        layers = [layer.clone() for layer in self.vectorLayers]
        layers.append(layerRaster)
        target.setLayers(layers)

        self.refCrs = layerRaster.crs()

        self.initProject()
        self.loadProject(self.mapViewerLayer.layers())
        
        if qgsMapLayerComboBox == self.listLayers : self.zoomToLayerExtent(layerRaster, target)

        if qgsMapLayerComboBox == self.listLayers :
          if self.listLayers.currentIndex()< 0 : return 
          targetLayer = self.listLayers.layer(self.listLayers.currentIndex())

          

    def synchronizeExtent(self):
        index  = self.listLayersDouble.currentIndex()
        self.mapViewerLayer.refresh()
        self.setExtent()
        self.synchronizeMap(index)


    def updateListLayersDouble(self):
          self.listLayersDouble.clear()
          if self.listLayers.count() == 0 : return
          for i in range(self.listLayers.count()):  self.listLayersDouble.addItem(self.rasterIcon, self.listLayers.itemText(i))
          self.synchronizeMap(self.listLayers.currentIndex())
  
    def synchronizeMap(self, indexTarget):
          if self.listLayers.count() == 0 : return
                
          self.layers, refLayer = [], None
        
          for i in range(self.listLayers.count()):
              self.listLayersDouble.setItemData(i, 0, Qt.UserRole-1)
              item = self.listLayersDouble.model().item(i, 0)
              if item : item.setCheckState(Qt.Unchecked)

          geomTargetExtent = QgsGeometry().fromRect(self.mapViewerLayer.extent())

          first = True
          newIndex = 0 if self.listLayersDouble.currentIndex() == -1 else self.listLayersDouble.currentIndex()
          ajustIndex, indexesGroupEnabled = False, []
          groupForTheLayer = self.getGroupForTheLayer(self.listLayers.layer(newIndex))

          
          for i in range(self.listLayers.count()):
              self.listLayersDouble.setItemText(i, self.listLayers.itemText(i))
              layer = self.listLayers.layer(i)

              geomExtent = QgsGeometry().fromRect(layer.extent())
              item = self.listLayersDouble.model().item(i, 0)

              if (geomExtent.overlaps(geomTargetExtent) or geomExtent.equals(geomTargetExtent) or geomExtent.intersects(geomTargetExtent)) :
                  self.listLayersDouble.setItemData(i, 33, Qt.UserRole-1)
                  if item : item.setCheckState(Qt.Checked)
                  
                  if first :  refLayer = layer; first = False 
                  if i == indexTarget : refLayer = layer 
                  self.layers.append(layer)

                  if self.getGroupForTheLayer(layer) == groupForTheLayer : indexesGroupEnabled.append(i) 
                  
              else : self.listLayersDouble.setItemData(i, 0, Qt.UserRole-1)

              if i == newIndex :
                 if item : 
                    if not item.isEnabled() :  ajustIndex = True  

          if ajustIndex :
             if indexesGroupEnabled != []: newIndex = indexesGroupEnabled[0]
              
          self.setLayers(refLayer)
          self.listLayersDouble.setCurrentIndex(newIndex)


    def getVectorLayer(self):      
        root = QgsProject.instance().layerTreeRoot()
        allLayers, allLayersIcon = [], []
        iconVector, iconRaster = QIcon(getThemeIcon("vector.png")),  QIcon(getThemeIcon("raster.png"))
        for layer in root.findLayers() :
            if type(layer.layer()) in (QgsVectorLayer, QgsRasterLayer) and not layer.layer() in self.vectorLayers :
               allLayers.append(layer.layer().name())
               allLayersIcon.append(iconVector if layer.layer().type() == 0 else iconRaster)
        return allLayers, allLayersIcon

    def delLayersFromMapCanvas(self):
        frozenLayer = self.listLayers.layer(self.listLayers.currentIndex())

        allLayers, allLayersIcon = [], []
        iconVector, iconRaster = QIcon(getThemeIcon("vector.png")),  QIcon(getThemeIcon("raster.png"))

        for layer in self.mapViewerLayer.layers() : 
            if type(layer) in (QgsVectorLayer, QgsRasterLayer) :
               allLayers.append(layer.name())
               allLayersIcon.append(iconVector if layer.type() == 0 else iconRaster)
        if allLayers == [] : return
        
        self.dialogInput = QInputDialog(self) 
        self.dialogInput.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin","Resources"))
        self.dialogInput.setLabelText("%s :" % QApplication.translate("autoLoaderRasterPlugin","Layers"))
        self.dialogInput.setOptions(QInputDialog.UseListViewForComboBoxItems)
        windowIcon(self.dialogInput, "del.png")

        self.dialogInput.resize(600, 500)
        self.dialogInput.setComboBoxItems(allLayers) 

        child = None
        for child in self.dialogInput.children():
            if type(child) == QListView :
                child.setIconSize(QSize(18, 18))
                child.setSelectionMode( QAbstractItemView.ExtendedSelection )

                model = child.model()
                for index in range(model.rowCount()):
                    listViewItem = model.item(index)
                    listViewItem.setData( allLayersIcon[index], Qt.DecorationRole)
                    if listViewItem.text() == frozenLayer.name() : listViewItem.setEnabled(False)
                    
                    
        

        if self.dialogInput.exec_() :
            if child :
               instance = self.project
               layersMapView = self.mapViewerLayer.layers()
               layersMapViewDouble = self.mapViewerLayerDouble.layers()
               
               for index in child.selectionModel().selectedIndexes() :
                   nameLayer = child.model().itemFromIndex(index).text()
                   layers = instance.mapLayersByName(nameLayer)
                   
                   for layer in layers :
                      
                      if layer in layersMapView : layersMapView.remove(layer)
                      if layer in layersMapViewDouble :  layersMapViewDouble.remove(layer)
                      if layer in self.vectorLayers : self.vectorLayers.remove(layer)
                      
                      self.project.removeMapLayers( [layer.id()] )

               self.mapViewerLayer.setLayers(layersMapView)
               self.mapViewerLayerDouble.setLayers(layersMapViewDouble)

               self.initProject()
               self.loadProject(self.mapViewerLayer.layers())
               self.zoomPrevious()

          
    def addLayersToMapCanvas(self):
        frozenLayer = self.listLayers.layer(self.listLayers.currentIndex())
        allLayers, allLayersIcon = self.getVectorLayer()
        if allLayers == [] : return
        
        self.dialogInput = QInputDialog(self) 
        self.dialogInput.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin","Resources"))
        self.dialogInput.setLabelText("%s :" % QApplication.translate("autoLoaderRasterPlugin","Layers"))
        self.dialogInput.setOptions(QInputDialog.UseListViewForComboBoxItems)
        windowIcon(self.dialogInput, "add.png")

        self.dialogInput.resize(600, 500)
        
        self.dialogInput.setComboBoxItems(allLayers) 
        icon = QIcon(getThemeIcon("vector.png"))

        child = None
        for child in self.dialogInput.children():
            if type(child) == QListView :
                child.setIconSize(QSize(18, 18))
                child.setSelectionMode( QAbstractItemView.ExtendedSelection )

                model = child.model()
                for index in range(model.rowCount()):
                    listViewItem = model.item(index)
                    listViewItem.setData( allLayersIcon[index], Qt.DecorationRole) 
                    if listViewItem.text() == frozenLayer.name() : listViewItem.setEnabled(False)

        if self.dialogInput.exec_() :
            if child :
               instance = QgsProject.instance() 
               layersMapView = self.mapViewerLayer.layers()
               layersMapViewDouble = self.mapViewerLayerDouble.layers()
               self.vectorLayers = []
               
               for index in child.selectionModel().selectedIndexes() :
                   nameLayer = child.model().itemFromIndex(index).text()
                   layer = instance.mapLayersByName(nameLayer)
                   
                   if layer : 
                      if type(layer)== list : layer = layer[0]

                      if  not layer in self.mapViewerLayer.layers() :
                          if layer.hasScaleBasedVisibility() : layer.setScaleBasedVisibility(False)
                          clayer = layer.clone()
                          layersMapView.insert(0, clayer)
                          if layer.type() != QgsRasterLayer :                           
                             layersMapViewDouble.insert(0, clayer)
                             self.vectorLayers.append(clayer)
                        
               self.mapViewerLayer.setLayers(layersMapView)
               self.initProject()
               self.loadProject(self.mapViewerLayer.layers())
               self.zoomPrevious()
                   
          
    def setLayers(self, refLayer):
        self.currentLayers = []
        groupRefLayer = self.getGroupForTheLayer(refLayer)
        for layer in self.layers :
            group = self.getGroupForTheLayer(layer)
            if group == groupRefLayer : self.currentLayers.append(layer)
        if groupRefLayer == None : groupRefLayer = refLayer    


    def getGroupForTheLayer(self, mapCanvasLayer):      
        root = QgsProject.instance().layerTreeRoot()
        groupLayer = None
        for layer in root.findLayers() :
            if layer.layer() == mapCanvasLayer :
               if type(layer.parent()) == QgsLayerTreeGroup : groupLayer = layer.parent() ; break
        return groupLayer
          

    def zoomToLayerExtent(self, layer, target):
        try : 
            ds = gdal.Open(layer.source())
        
            minX, maxX = ds.GetGeoTransform()[0], (ds.RasterXSize*ds.GetGeoTransform()[1])+ds.GetGeoTransform()[0] 
            maxY, minY = ds.GetGeoTransform()[3], ds.GetGeoTransform()[3]+(ds.RasterYSize*ds.GetGeoTransform()[5])
            ds = None

            target.setExtent(QgsRectangle(minX, minY, maxX, maxY))
            target.refresh()
            
        except : return    
       


def actionsWindows(parent, name, title, width, height):
        self = QMdiSubWindow(parent)
        self.setObjectName(name)
        self.setWindowTitle(title)
        
        self.setWindowFlags(Qt.WindowTitleHint | Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint)
        self.setFixedSize(width, height)
        self.setGeometry(10, 10, width, height)

        self.setStyleSheet("QMdiSubWindow { border: 1px solid;    border-radius: 5px;    opacity: 0.5;    background-color: rgba(255, 255, 255, 50);}"
                           "QMdiSubWindow QWidget { background-color: transparent; border:1px;}"
                           "QMdiSubWindow:title { height: 18px; background-color: rgba(255, 255, 255, 50);}")
        self.oldPos = self.pos()

        return self

def ctrlMap(self, conteneur, complete, getPointsForExtent, simply, iX, iY, iW, iH, allActions):
        
         self.groupMap = QGroupBox(conteneur)

         self.actionsMapCanvas = QButtonGroup(conteneur) 
         self.actionsMapCanvas.setExclusive(True)

         if allActions :

             dicoCtrls = {"buttonCatchExtent" : ("catchextent.png", "Get extent for QGIS mapcanvas", 10, 20),
                          "buttonZoomIn" : ("zoomin.png", "Zoom in the map window", 40, 20),
                          "buttonZoomOut" : ("zoomout.png", "Zoom out the map window", 70, 20),
                          "buttonZoomAll" : ("zoomall.png", "Zoom all the map window", 100, 20),
                          "buttonZoomCurrentLayer" : ("zoomcurrentlayer.png", "Zoom to grid layer", 130, 20),
                          "buttonZoomPrevious" : ("zoomprev.png", "Previous zoom", 160, 20),
                          "buttonZoomNext" : ("zoomnext.png", "Next zoom", 190, 20),
                          "buttonSendExtent" : ("sendextent.png", "Send extent to Qgis mapcanvas", 220, 20),
                          "buttonPan" : ("pan.png", "Move the map window", 250, 20),
                          "buttonHotlink" : ("hotlink.png", "Load a raster file", 280, 20),
                          "buttonLoadExtent" : ("hotlinkextent.png", "Load the raster files for extent", 310, 20),
                          "buttonLoadSelected" : ("hotlinkselected.png", "Load the raster files for selected objects", 340, 20)
                          }
             

         else :
             dicoCtrls = {"buttonCatchExtent" : ("catchextent.png", "Get extent for QGIS mapcanvas", 10, 20),
                          "buttonZoomIn" : ("zoomin.png", "Zoom in the map window", 40, 20),
                          "buttonZoomOut" : ("zoomout.png", "Zoom out the map window", 70, 20),
                          "buttonZoomAll" : ("zoomall.png", "Zoom to all ...", 100, 20),
                          "buttonZoomPrevious" : ("zoomprev.png", "Previous zoom", 130, 20),
                          "buttonZoomNext" : ("zoomnext.png", "Next zoom", 160, 20),
                          "buttonSendExtent" : ("sendextent.png", "Send extent to Qgis mapcanvas", 190, 20),
                          "buttonPan" : ("pan.png", "Move the map window", 220, 20),
                          "buttonAddPermanentLayers" : ("add.png", "Add permanent vector layers", 250, 20),
                          "buttonDelPermanentLayers" : ("del.png", "Delete permanent vector layers", 280, 20)
                      }


         pos = 5

       
         for key in dicoCtrls :
      
                   icon = getThemeIcon(dicoCtrls[key][0])
                   toolTip = QApplication.translate("autoLoaderRasterPlugin", dicoCtrls[key][1])

                   button = myPushButton(self.groupMap)
                   
                   if key!= "buttonHotlink" : button.initPushButton(25, 25, pos, dicoCtrls[key][3], key, "", toolTip, True, icon, 25, 25, True)
                   else :  button.initPushButton(50, 25, pos, dicoCtrls[key][3], key, "", toolTip, True, icon, 50, 25, True)

                   if key in ("buttonZoomIn", "buttonZoomOut","buttonPan") : 

                      button.setStyleSheet(self.styleSheetPushButton)
                      button.setCheckable(True)

                      if key == "buttonZoomIn" : button.setChecked(True)
 
                      index = ("buttonZoomIn", "buttonZoomOut","buttonPan").index(key)
                      self.actionsMapCanvas.addButton(button, index)
                      

                   if key == "buttonHotlink" :
                      button.setStyleSheet(self.styleSheetPushButtonWithMenu)
                      button.setCheckable(True)

                      self.contextMnuToolsHotlink(button)
                   
                   setattr(self, key, button)
                   
                   if key == "buttonHotlink" : pos+= 50
                   else : pos+= 30 
                   

         sizeWidth = len(self.groupMap.children())*30
         self.groupMap.setGeometry(5, 10, sizeWidth+20, 50)           

         return sizeWidth


