from qgis.PyQt.QtCore import (QFile, QSize, Qt)
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import (QApplication, QFileDialog, QLabel, QPushButton, QProgressBar, QTextEdit)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.core import (Qgis, QgsApplication)
from os.path import dirname
from sys import path as sysPath
from xml.dom.minidom import parseString

try:
    from osgeo.gdal import Open as gdalopen
    from osgeo.gdal import GetDriver as gdal_GetDriver
    from osgeo.gdal import GetDriverCount as gdal_GetDriverCount
    from osgeo.gdal import GetDriverByName as gdal_GetDriverByName
    from osgeo.gdal import (DMD_LONGNAME, DMD_MIMETYPE, DMD_EXTENSION)
    from osgeo.ogr import GetDriver as ogr_GetDriver
    from osgeo.ogr import GetDriverCount as ogr_GetDriverCount
except:
    from gdal import Open as gdalopen
    from gdal import GetDriver as gdal_GetDriver
    from gdal import GetDriverCount as gdal_GetDriverCount
    from gdal import GetDriverByName as gdal_GetDriverByName
    from gdal import (DMD_LONGNAME, DMD_MIMETYPE, DMD_EXTENSION)
    from ogr import GetDriver as ogr_GetDriver
    from ogr import GetDriverCount as ogr_GetDriverCount

def makeListFormatsExtensions(self, vector, raster, includeJoker):
    listFormatsExtensions, listFormatsExtensionsVector, listFormatsExtensionsAll = [], [], []

    for iDriver in range(ogr_GetDriverCount()):
        poDriver = ogr_GetDriver(iDriver)
        poValue = poDriver.GetName()
        driver = gdal_GetDriverByName("%s" % (poValue))
        listFormatsExtensionsVector = setEltListFormatsExtensions(self, listFormatsExtensionsVector, driver, poValue)
        

    for iDriver in range(gdal_GetDriverCount()):
        poDriver = gdal_GetDriver(iDriver)
        poValue = poDriver.ShortName
        driver = gdal_GetDriverByName("%s" % (poValue))
        listFormatsExtensionsAll = setEltListFormatsExtensions(self, listFormatsExtensionsAll, driver, poValue)

    if vector and raster : listFormatsExtensions =  listFormatsExtensionsAll
    else :
        if vector : listFormatsExtensions = listFormatsExtensionsVector
        elif raster :
            listFormatsExtensions = [value for value in listFormatsExtensionsAll if value not in listFormatsExtensionsVector] 
            if not "jp2" in listFormatsExtensions : listFormatsExtensions.append('jp2')
            
    if includeJoker : listFormatsExtensions.append('*')
    return sorted(listFormatsExtensions)

def setEltListFormatsExtensions(self, listFormatsExtensions, driver, poValue):
       if driver :
            poDriverMetadata = driver.GetMetadata()
            if dicoHasKey(poDriverMetadata, DMD_EXTENSION):
                if poDriverMetadata[DMD_EXTENSION]!= '' and not poDriverMetadata[DMD_EXTENSION] in  listFormatsExtensions : listFormatsExtensions.append(poDriverMetadata[DMD_EXTENSION])
       else :
           if poValue != '' and not poValue in  listFormatsExtensions : listFormatsExtensions.append[poValue]
       return listFormatsExtensions


def getExtension(fileName):
    from os.path import (basename, splitext)
    textension = splitext(fileName)
    extension = textension[len(textension)-1].lower()
    return extension

def windowIcon(qtWindow, icon):
    if not qtWindow : return
    icon = getThemeIcon(icon)
    if icon != "" : qtWindow.setWindowIcon(QIcon(icon)) 

def changeButtonIcon(self, button, icon, iconWidth, iconHeight):
    icon = getThemeIcon(icon)
    button.setIcon(QIcon(icon))

def getThemeIcon(theName):
    dirs = list(sysPath)
    dirs.insert(0, "%s%s" % (cleanPath(dirname(__file__)),"icons/"))
    dirs.insert(0, "%s%s" % (cleanPath(QgsApplication.pluginPath()),"autoLoaderRaster/icons/"))
    for dir in dirs :
        myPath = cleanPath(dir)
        myIconPath = "%s%s" % (myPath, theName)
        if QFile.exists(myIconPath): return myIconPath
    return ""
    
def cleanPath(path):
    path = "%s" % (path)
    a = len(path)
    letter = "/"
    b = path.rfind(letter, 0, a)
    return ("%s/" % (path)) if a != b else path 
    
def fixeLabelsFileDialog(self, dlgFile, context, hasName):
    dlgFile.setLabelText(QFileDialog.Accept, QApplication.translate("autoLoaderRasterPlugin","Open")) if context == 0 else dlgFile.setLabelText(QFileDialog.Accept, QApplication.translate("autoLoaderRasterPlugin","Save"))
    dlgFile.setLabelText(QFileDialog.Reject, QApplication.translate("autoLoaderRasterPlugin","Cancel"))        
    dlgFile.setLabelText(QFileDialog.FileName, "%s : " % (QApplication.translate("autoLoaderRasterPlugin","Name"))) if hasName else dlgFile.setLabelText(QFileDialog.FileName, QApplication.translate("autoLoaderRasterPlugin","Folder : "))
    dlgFile.setLabelText(QFileDialog.FileType, "%s : " % QApplication.translate("autoLoaderRasterPlugin","Filters"))
    dlgFile.setLabelText(QFileDialog.LookIn,   "%s : " % QApplication.translate("autoLoaderRasterPlugin","Folder"))

def dicoHasKey(dico, key):
    cond = False
    try : cond = dico.has_key(key)
    except : cond = (key in dico)
    return cond

def prettifyXml(xml):
    return parseString(xml).toprettyxml()


class textEditFolder(QTextEdit):
    def __init__(self, * args):
        QTextEdit.__init__(self, * args)

    def initTextEdit(self, SizeW, SizeH, posX, posY, name, isGeom, undoRedo, isReadOnly, isRichText, isSizable):
        if not isSizable:  self.setMaximumSize(SizeW, SizeH)
        if isGeom: self.setGeometry(posX, posY, SizeW, SizeH)
        self.setUndoRedoEnabled(undoRedo)
        self.setReadOnly(isReadOnly)
        self.setAcceptRichText(isRichText)
        self.setObjectName(name)
        self.setAccessibleName(name)
        self.setAcceptDrops(True)

class editor(QTextEdit):
    def __init__(self, parent, withProgressBar):
        super(editor, self).__init__(parent)
        self.setWindowTitle("QTextEdit Standard Output Redirection")
        self.setStyleSheet("color: white; background-color: black;")
        self.setAcceptRichText(True)
        self.setObjectName(parent.objectName())
        self.parent = parent
        self.progressBar = None

        if withProgressBar :
            self.progressBar = progressBar(self)
            self.progressBar.initTextEdit(200, 25, 200, 140, "progressBar", True, 0, True, Qt.AlignCenter, 0)
            self.progressBar.setStyleSheet("progressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
                                            "progressBar::chunk {background-color: #6C96C6; width: 20px;}")


class funnyPushButton(QPushButton):
     def __init__(self, *args): super(funnyPushButton, self).__init__(*args)
            
     def initPushButton(self, sizeWidth, sizeHeight, coordX, coordY, name, text, toolTip, isGeom, icon, iconWidth, iconHeight, isStyleSheeted):
            self.setMinimumSize(sizeWidth, sizeHeight)
            self.setMaximumSize(sizeWidth, sizeHeight)
            self.iconWidth = iconWidth
            self.iconHeight = iconHeight
            self.selfFocused = False
            self.subMenuVisble = False
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            if icon != "" :
               self.setIcon(QIcon(icon))
               self.setIconSize(QSize(self.iconWidth , self.iconHeight))
            self.setToolTip(toolTip)
            if isStyleSheeted : self.setStyleSheet(" QPushButton {border-width: 0px; border-radius: 10px;  border-color: beige;}")
            self.setObjectName(name)
            if text != "" : self.setText(text)

     def enterEvent(self, event):
          if not self.selfFocused :
             self.setIconSize(QSize(self.iconWidth *0.8, self.iconHeight *0.8))
             self.selfFocused = True
          
     def leaveEvent(self, event):
        if self.subMenuVisble == False : self.restoreButton()
     
     def restoreButton(self):
           self.selfFocused = False    
           self.setIconSize(QSize(self.iconWidth, self.iconHeight))
           
      
class labeller(QLabel):
     def __init__(self, name, text, styleSheet, alignment, *args):
             super(labeller, self).__init__(*args)
             self.setObjectName(name)
             self.setText(text)
             if styleSheet == "special": self.setStyleSheet("QLabel {display: block;  border: 2px solid gray;  border-radius: 10px; text-align: center;  line-height: 10px; background-color: #C0C0C0; border: 1px solid black; color: black}")
             elif styleSheet == "infos": self.setStyleSheet("QLabel {font-size:10px; background-color:#C0C0C0; border: 1px solid #e1e1e1; padding: 4px ; color: black}")
             elif styleSheet == "coords": self.setStyleSheet("QLabel {  color : white; font-size: 11px; background-color: rgba(93, 93, 93, 125);}")
             self.setAlignment(alignment)

class myPushButton(QPushButton):
     def __init__(self, *args): super(myPushButton, self).__init__(*args)
            
     def initPushButton(self, sizeWidth, sizeHeight, coordX, coordY, name, text, toolTip, isGeom, icon, iconWidth, iconHeight, isStyleSheeted):
            self.setMinimumSize(sizeWidth, sizeHeight)
            self.setMaximumSize(sizeWidth, sizeHeight)
            if isGeom : self.setGeometry(coordX, coordY, sizeWidth, sizeHeight)
            if icon != "" :
               self.setIcon(QIcon(icon))
               self.setIconSize(QSize(iconWidth, iconHeight))
            self.setToolTip(toolTip)
            if isStyleSheeted : self.setStyleSheet("QPushButton {border: none;}")
            self.setObjectName(name)
            if text != "" : self.setText(text)

class progressBar(QProgressBar):
    def __init__(self, * args):
        QProgressBar.__init__(self, * args)

    def initTextEdit(self, width, heigth, posX, posY, name, isGeom, initValue, viewText, alignement, isStyleSheet):
        self.setMinimumSize(width, heigth)
        self.setMaximumSize(width, heigth)
        if isGeom: self.setGeometry(posX, posY, width, heigth)
        self.setProperty("value", initValue)
        self.setAlignment(alignement)
        self.setTextVisible(viewText)
        self.setObjectName(name)
        self.setAccessibleName(name)
        if isStyleSheet == 0:
            self.setStyleSheet(
                               "progressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
                               "progressBar::chunk {background-color: #6C96C6; width: 10px; margin: 0.5px;}"
                               )            


class printerDlg(object):
      def __init__(self, parent, obj, title, orientation):
                    
              printer = QPrinter() 
              printer.setPageSize(QPrinter.A4)
              printer.setOrientation(orientation)
              printer.setPageMargins(5, 10, 5, 10, QPrinter.Millimeter)
              printer.setOrientation(orientation)
              printer.setOutputFormat(QPrinter.NativeFormat)
              self = QPrintPreviewDialog(printer)
              windowIcon(self, "print.png")
              self.setWindowTitle(title)
              self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowStaysOnTopHint | Qt.WindowCloseButtonHint)
              self.paintRequested.connect(obj.print_)
              self.exec_()
